require 'progressbar.rb'
class Treemaker
  @model = Sketchup.active_model

  def Treemaker.dialog
  
    prompts  = ["Branches:","Minimum number of turns per branch: ", "Maximum number of turns per branch: ","Enable transparency?"]
    defaults = ["50", "10", "25", "No"]
    list     = ["", "", "", "Yes|No"]
    input    = UI.inputbox prompts, defaults, list, "Create a tree"
    valid    = 1
    tryagain = 0
    if input
      # Check for invalid values
      if input[1].to_i > input[2].to_i
        valid  = 0
        result = UI.messagebox "The minimum number of turns per branch must be SMALLER THAN OR EQUAL TO the maximum number of turns per branch.\nTry again?", MB_YESNO
        if result == 6 # Yes
          tryagain = 1
        end
      end
      
      if input[1].to_i <= 0
        valid  = 0
        result = UI.messagebox "The minimum number of turns per branch must be LARGER THAN 0.\nTry again?", MB_YESNO
        if result == 6 # Yes
          tryagain = 1
        end
      end
      
      if input[2].to_i <= 0
        valid  = 0
        result = UI.messagebox "The maximum number of turns per branch must be LARGER THAN 0.\nTry again?", MB_YESNO
        if result == 6 # Yes
          tryagain = 1
        end
      end
      
      if input[0].to_i <= 0
        valid  = 0
        result = UI.messagebox "The number of branches must be LARGER THAN 0.\nTry again?", MB_YESNO
        if result == 6 # Yes
          tryagain = 1
        end
      end
      
      if tryagain == 1
        Treemaker.dialog
      elsif valid == 1
        @model.start_operation("Make a tree")
        Treemaker.maketree(input[0].to_i, input[1].to_i, input[2].to_i, input[3].to_s)
        @model.commit_operation
      end
    end
  end
  
  def Treemaker.maketree(branches, minturns, maxturns, transparency)
  
    entities1 = @model.active_entities.add_group.entities
    materials = @model.materials
    progress  = ProgressBar.new(branches, "of #{branches} branches have been made." )

    numb = 0
    until numb == branches
      entities = entities1.add_group.entities
      
      x    = 0
      y    = 0
      z    = 0
      h    = rand(10) + 5
      tree = [[0, 0, 0]]
      z    = z + rand(20) + 190
      tree.push [x, y, z]
      
      randturns = rand(maxturns-minturns)+minturns

      if minturns == maxturns
        randturns = maxturns
      end
      
      (1..randturns).each { |i|
        x = x + rand(6*i) - 3*i
        y = y + rand(6*i) - 3*i
        z = z + h
        tree.push [x, y, z]
      }
      
      curve       = entities.add_curve tree
      centerpoint = Geom::Point3d.new(0,0,0)
      vector      = Geom::Vector3d.new 0,0,1
      vector2     = vector.normalize!
      circle      = entities.add_circle centerpoint, vector2, (rand(4)+1), 4
      face        = entities.add_face circle
      color       = face.material = face.back_material = [152,86,42]
      branch      = face.followme curve
      edge        = curve[0]
      curve       = edge.curve
      edges       = curve.edges

      edges.each{ |e|
        e.erase!
      }

      entities    = entities1.add_group.entities
      centerpoint = Geom::Point3d.new(x,y,z)
      vector      = Geom::Vector3d.new 1,0,0
      vector2     = vector.normalize!
      circle      = entities.add_circle centerpoint, vector2, (rand(50)+25), 12
      face        = entities.add_face circle
      if transparency == "Yes"
        mat           = materials.add "Green"
        color         = mat.color          = "Green"
        alpha         = mat.alpha          = (rand(50))/100.0
        face.material = face.back_material = mat
      else
        face.material = face.back_material = [0,127,0]
      end

      centerpoint = Geom::Point3d.new(x,y,0)
      vector      = Geom::Vector3d.new 0,0,1
      vector2     = vector.normalize!
      circle      = entities.add_circle centerpoint, vector2, 1, 12
      sphere      = face.followme circle
      edge        = circle[0]
      curve       = edge.curve
      edges       = curve.edges

      edges.each{ |e|
      e.erase!
      }
      numb = numb+1
      progress.update(numb)
    end
  end
end

UI.menu('Plugins').add_item('Treemaker') {Treemaker.dialog}
